@@ -192,59 +192,10 @@ class LiveCDCopyBackend(backend.AnacondaBackend):
         log.info("Do we need to run GRUB2 setup? => %s" % (self._install_grub,))
 
         if self._install_grub:
-
             # HACK: since Anaconda doesn't support grub2 yet
             # Grub configuration is disabled
             # and this code overrides it
-            encrypted, root_crypted, swap_crypted = self._setup_grub2()
-            swap_dev_name_changed = False
-            if encrypted:
-                swap_dev = None
-                root_dev = None
-                if swap_crypted:
-                    # in case of encrypted swap_dev sitting on top of
-                    # LVMLogicalVolumeDevice, do not force fstabSpec
-                    # to /dev/mapper/swap, otherwise fstab will end up
-                    # having wrong devspec.
-                    swap_name, swap_dev = swap_crypted
-                    if not isinstance(swap_dev,
-                                  storage.devices.LVMLogicalVolumeDevice):
-                        old_swap_name = swap_dev._name
-                        swap_dev._name = swap_name
-                        swap_dev_name_changed = True
-
-                if root_crypted:
-                    root_name, root_dev = root_crypted
-                    old_root_name = root_dev._name
-                    if root_name == "root": # comes from /dev/mapper/root
-                        root_dev._name = root_name
-
-                def _crypt_filter_callback(cb_dev):
-                    # this is required in order to not get
-                    # crypt root device and crypt swap written
-                    # into /etc/conf.d/dmcrypt, as per bug #2522
-                    handled_devs = [swap_dev, root_dev]
-                    # use is and not ==, so, loop manually
-                    for handled_dev in handled_devs:
-                        if cb_dev is handled_dev:
-                            # we are already handling this device,
-                            # so skip it
-                            return False
-                        if handled_dev is not None:
-                            if handled_dev.dependsOn(cb_dev):
-                                # already handling this device, root dev?
-                                return False
-                    return True
-
-                # HACK: since swap device path value is potentially changed
-                # it is required to rewrite the fstab
-                # (circular dependency, sigh)
-                self.anaconda.storage.fsset.write(
-                    crypt_filter_callback=_crypt_filter_callback)
-                if swap_crypted and swap_dev_name_changed:
-                    swap_dev._name = old_swap_name
-                if root_crypted:
-                    root_dev._name = old_root_name
+            self._setup_grub2()
 
         self._copy_logs()
         # also remove hw.hash
@@ -322,6 +273,7 @@ class LiveCDCopyBackend(backend.AnacondaBackend):
 
         # use reference, yeah
         cmdline = self._sabayon_install.cmdline
+        final_cmdline = []
 
         # Sabayon MCE install -> MCE support
         if Entropy.is_sabayon_mce() and ("sabayonmce" not in cmdline):
@@ -355,8 +307,13 @@ class LiveCDCopyBackend(backend.AnacondaBackend):
         if "domdadm" not in cmdline:
             cmdline.append("domdadm")
 
+        # setup LVM
+        lvscan_out = commands.getoutput("LANG=C LC_ALL=C lvscan").split(
+            "\n")[0].strip()
+        if not lvscan_out.startswith("No volume groups found"):
+            final_cmdline.append("dolvm")
+
         previous_vga = None
-        final_cmdline = []
         for arg in cmdline:
             for check in ourargs:
                 if arg.startswith(check):
@@ -374,142 +331,74 @@ class LiveCDCopyBackend(backend.AnacondaBackend):
         # device.format.options, device.path, device.fstabSpec
         root_crypted = False
         swap_crypted = False
-        delayed_crypt_swap = None
-        any_crypted = len(fsset.cryptTab.mappings.keys()) > 0
-
-        if swap_devices:
-            log.info("Found swap devices: %s" % (swap_devices,))
-            swap_dev = swap_devices[0]
-
-            swap_crypto_dev = None
-            for name in fsset.cryptTab.mappings.keys():
-                swap_crypto_dev = fsset.cryptTab[name]['device']
-                if swap_dev == swap_crypto_dev or swap_dev.dependsOn(
-                    swap_crypto_dev):
-                    swap_crypted = True
-                    break
-
-            if swap_crypted:
-                # genkernel hardcoded bullshit, cannot change /dev/mapper/swap
-                # change inside swap_dev, fstabSpec should return
-                # /dev/mapper/swap
-                swap_crypted = ("swap", swap_dev)
-                # if the swap device is on top of LVM LV device, don't
-                # force /dev/mapper/swap, because it's not going to work
-                # with current genkernel.
-                if not isinstance(swap_dev,
-                                  storage.devices.LVMLogicalVolumeDevice):
-                    final_cmdline.append("resume=swap:/dev/mapper/swap")
-                    final_cmdline.append("real_resume=/dev/mapper/swap")
-                else:
-                    final_cmdline.append("resume=swap:%s" % (
-                            swap_dev.fstabSpec,))
-                    final_cmdline.append("real_resume=%s" % (
-                            swap_dev.fstabSpec,))
-                # NOTE: cannot use swap_crypto_dev.fstabSpec because
-                # genkernel doesn't support UUID= on crypto
-                delayed_crypt_swap = swap_crypto_dev.path
-            else:
-                final_cmdline.append("resume=swap:%s" % (swap_dev.fstabSpec,))
-                final_cmdline.append("real_resume=%s" % (swap_dev.fstabSpec,))
-
-        # setup LVM
-        lvscan_out = commands.getoutput("LANG=C LC_ALL=C lvscan").split(
-            "\n")[0].strip()
-        if not lvscan_out.startswith("No volume groups found"):
-            final_cmdline.append("dolvm")
-
         crypto_dev = None
+        swap_crypto_dev = None
+
         for name in fsset.cryptTab.mappings.keys():
             crypto_dev = fsset.cryptTab[name]['device']
             if root_device == crypto_dev or root_device.dependsOn(crypto_dev):
                 root_crypted = True
                 break
 
-        # - in case of real device being crypted, crypto_dev is
-        #   a storage.devices.PartitionDevice object
-        # - in case of crypt over lvm, crypto_dev is
-        #   storage.devices.LVMLogicalVolumeDevice
-
-        # crypt over raid?
-
-        def is_parent_a_simple_device(root_device):
-            if not hasattr(root_device, 'parents'):
-                return False
-            for parent in root_device.parents:
-                if not isinstance(parent, storage.devices.PartitionDevice):
-                    return False
-            return True
-
-        def is_parent_a_md_device(root_device):
-            if not hasattr(root_device, 'parents'):
-                return False
-            for parent in root_device.parents:
-                if isinstance(parent, storage.devices.MDRaidArrayDevice):
-                    return True
-            return False
-
-        def is_parent_a_lv_device(root_device):
-            if not hasattr(root_device, 'parents'):
-                return False
-            for parent in root_device.parents:
-                if isinstance(parent, storage.devices.LVMVolumeGroupDevice):
-                    return True
-            return False
-
-        def translate_real_root(root_device, crypted):
-            # crypt over anything, == "/dev/mapper/root"
-            if crypted and isinstance(root_device, storage.devices.LUKSDevice):
-                return "/dev/mapper/root"
-            if crypted and is_parent_a_md_device(root_device):
-                return "/dev/mapper/root"
-            if crypted and is_parent_a_simple_device(root_device):
-                return "/dev/mapper/root"
-
-            # not needed anymore with grub 1.99
-            # if isinstance(root_device, storage.devices.MDRaidArrayDevice):
-            #    return root_device.path
-            return root_device.fstabSpec
-
         crypt_root = None
         if root_crypted:
             log.info("Root crypted? %s, %s, crypto_dev: %s" % (root_crypted,
                 root_device.path, crypto_dev.fstabSpec))
-
-            translated_real_root = translate_real_root(root_device, True)
-            root_crypted = (os.path.basename(translated_real_root), root_device)
             # must use fstabSpec now, since latest genkernel supports it
-            final_cmdline.append("root=%s crypt_root=%s" % (
-                translated_real_root, crypto_dev.fstabSpec,))
-            # due to genkernel initramfs stupidity, when crypt_root = crypt_swap
-            # do not add crypt_swap.
-            if delayed_crypt_swap == crypto_dev.path:
-                delayed_crypt_swap = None
+            final_cmdline.append("crypt_root=%s" % (crypto_dev.fstabSpec,))
 
-        else:
-            log.info("Root crypted? Nope!")
-            final_cmdline.append("root=%s" % (
-                translate_real_root(root_device, False),))
+        if swap_devices:
+            log.info("Found swap devices: %s" % (swap_devices,))
+            # TODO(lxnay): support multiple crypt_swap=
+            swap_dev = swap_devices[0]
+
+            for name in fsset.cryptTab.mappings.keys():
+                swap_crypto_dev = fsset.cryptTab[name]['device']
+                swap_depends = swap_dev.dependsOn(swap_crypto_dev)
+                log_s = "Checking cryptTab name=%s, swap_crypto_dev=%s {%s}, "
+                log_s += "crypto_dev=%s {%s}, swap_dev.dependsOn(swap)=%s"
+                log.info(log_s % (
+                        name, swap_crypto_dev, swap_crypto_dev.fstabSpec,
+                        crypto_dev, crypto_dev.fstabSpec, swap_depends))
+                if swap_dev == swap_crypto_dev or swap_depends:
+                    swap_crypted = True
+                    break
+
+            log.info("swap_crypted set to %s" % (swap_crypted,))
+
+            # Use .path instead of fstabSpec for cmdline because
+            # genkernel must create an appropriate device node
+            # inside /dev/mapper/ that starts with luks-<UUID>
+            # so that the generic /dev/mapper/swap will not be used
+            # and systemd won't shit in its pants.
+            final_cmdline.append("resume=%s" % (swap_dev.path,))
+            if swap_crypted:
+                add_crypt_swap = True
+                if root_crypted and crypto_dev is not None:
+                    if crypto_dev.fstabSpec == swap_crypto_dev.fstabSpec:
+                        # due to genkernel initramfs stupidity,
+                        # when crypt_root = crypt_swap
+                        # do not add crypt_swap.
+                        add_crypt_swap = False
+
+                if add_crypt_swap:
+                    # must use fstabSpec now, since latest genkernel supports it
+                    final_cmdline.append(
+                        "crypt_swap=%s" % (swap_crypto_dev.fstabSpec,))
+                else:
+                    log.info("Not adding crypt_swap= because "
+                             "crypto_dev == swap_crypto_dev")
 
         # always add docrypt, loads kernel mods required by cryptsetup devices
         if "docrypt" not in final_cmdline:
             final_cmdline.append("docrypt")
 
-        if delayed_crypt_swap:
-            final_cmdline.append("crypt_swap=%s" % (delayed_crypt_swap,))
-
         log.info("Generated boot cmdline: %s" % (final_cmdline,))
-
-        return final_cmdline, root_crypted, swap_crypted, any_crypted
+        return final_cmdline
 
     def _setup_grub2(self):
 
-        cmdline_args, root_crypted, swap_crypted, any_crypted = \
-            self._get_bootloader_args()
-
-        log.info("_setup_grub2, cmdline_args: %s | "
-            "root_crypted: %s | swap_crypted: %s" % (cmdline_args,
-            root_crypted, swap_crypted,))
+        cmdline_args = self._get_bootloader_args()
 
         # "sda" <string>
         grub_target = self.anaconda.bootloader.getDevice()
@@ -528,7 +417,6 @@ class LiveCDCopyBackend(backend.AnacondaBackend):
         self._write_grub2(cmdline_str, grub_target)
         # disable Anaconda bootloader code
         self.anaconda.bootloader.defaultDevice = -1
-        return root_crypted or swap_crypted, root_crypted, swap_crypted
 
     def _write_grub2(self, cmdline, grub_target):
 
